MODULE icestp
   !!======================================================================
   !!                       ***  MODULE icestp   ***
   !!   Sea-Ice model : LIM Sea ice model time-stepping
   !!======================================================================
#if defined key_ice_lim
   !!----------------------------------------------------------------------
   !!   'key_ice_lim' :                                   Lim sea-ice model
   !!----------------------------------------------------------------------
   !!   ice_stp       : sea-ice model time-stepping
   !!----------------------------------------------------------------------
   USE dom_oce
   USE oce  ! dynamics and tracers variables
   USE in_out_manager
   USE ice_oce         ! ice variables
   USE flx_oce         ! forcings variables
   USE dom_ice
   USE daymod
   USE phycst          ! Define parameters for the routines
   USE taumod
   USE ice
   USE iceini
   USE ocesbc
   USE lbclnk
   USE limdyn
   USE limtrp
   USE limthd
   USE limflx
   USE limdia
   USE limwri
   USE limrst

   IMPLICIT NONE
   PRIVATE

   !! * Routine accessibility
   PUBLIC ice_stp  ! called by step.F90

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!-----------------------------------------------------
   !!   LIM 2.0 , UCL-LODYC-IPSL  (2003)
   !!-----------------------------------------------------

CONTAINS

   SUBROUTINE ice_stp ( kt )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE ice_stp  ***
      !!                   
      !! ** Purpose :   Louvain la Neuve Sea Ice Model time stepping 
      !!
      !! ** Action  : - call the ice dynamics routine 
      !!              - call the ice advection/diffusion routine 
      !!              - call the ice thermodynamics routine 
      !!              - call the routine that computes mass and 
      !!                heat fluxes at the ice/ocean interface
      !!              - save the outputs 
      !!              - save the outputs for restart when necessary
      !!
      !! History :
      !!   1.0  !  99-11  (M. Imbard)  Original code
      !!        !  01-03  (D. Ludicone, E. Durand, G. Madec) free surf.
      !!   2.0  !  02-09  (G. Madec, C. Ethe)  F90: Free form and module
      !!----------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( in ) ::   kt         ! ocean time-step index

      !! * Local declarations
      INTEGER   ::   ji, jj   ! dummy loop indices

      REAL(wp) , DIMENSION(jpi,jpj)    :: &
         zsss_io, zsss2_io, zsss3_io          ! tempory workspaces
      !!----------------------------------------------------------------------

      IF( kt == nit000 ) THEN
# if defined key_coupled
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'ice_stp : Louvain la Neuve Ice Model (LIM)'
         IF(lwp) WRITE(numout,*) '~~~~~~~   coupled case'
#else
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'ice_stp : Louvain la Neuve Ice Model (LIM)' 
         IF(lwp) WRITE(numout,*) '~~~~~~~   forced case using bulk formulea'
#endif
         !  Initialize fluxes fields
         gtaux(:,:) = 0.e0
         gtauy(:,:) = 0.e0
      ENDIF

      ! Temperature , salinity and horizonta wind
      ! sst_io  and sss_io, u_io and v_io  are initialized at nit000 in limistate.F90 (or limrst.F90) with :
      !              sst_io = sst_io + (nfice - 1) * (tn(:,:,1)+rt0 )
      !              sss_io = sss_io + (nfice - 1) * sn(:,:,1)
      !              u_io  = u_io  + (nfice - 1) * 0.5 * ( un(ji-1,jj  ,1) + un(ji-1,jj-1,1) )
      !              v_io  = v_io  + (nfice - 1) * 0.5 * ( vn(ji  ,jj-1,1) + vn(ji-1,jj-1,1) )
      !    cumulate fields
      !
      sst_io(:,:) = sst_io(:,:) + tn(:,:,1) + rt0
      sss_io(:,:) = sss_io(:,:) + sn(:,:,1)

 
      ! vectors at F-point
      DO jj = 2, jpj
         DO ji = fs_2, jpi   ! vector opt.
            u_io(ji,jj) = u_io(ji,jj) + 0.5 * ( un(ji-1,jj  ,1) + un(ji-1,jj-1,1) )
            v_io(ji,jj) = v_io(ji,jj) + 0.5 * ( vn(ji  ,jj-1,1) + vn(ji-1,jj-1,1) )
         END DO
      END DO
      
      
      IF( MOD( kt-1, nfice ) == 0 ) THEN
         
         ! The LIM model is going to be call
         sst_io(:,:) = sst_io(:,:) / FLOAT( nfice ) * tmask(:,:,1)
         sss_io(:,:) = sss_io(:,:) / FLOAT( nfice )

         ! stress from ocean U- and V-points to ice U,V point
         DO jj = 2, jpj
            DO ji = fs_2, jpi   ! vector opt.
               gtaux(ji,jj) = 0.5 * ( taux(ji-1,jj  ) + taux(ji-1,jj-1) )
               gtauy(ji,jj) = 0.5 * ( tauy(ji  ,jj-1) + tauy(ji-1,jj-1) )
               u_io  (ji,jj) = u_io(ji,jj) / FLOAT( nfice )
               v_io  (ji,jj) = v_io(ji,jj) / FLOAT( nfice )
            END DO
         END DO

         ! lateral boundary condition
         CALL lbc_lnk( gtaux(:,:), 'I', -1. )   ! I-point (i.e. ice U-V point)
         CALL lbc_lnk( gtauy(:,:), 'I', -1. )   ! I-point (i.e. ice U-V point)
         CALL lbc_lnk( u_io (:,:), 'I', -1. )   ! I-point (i.e. ice U-V point)
         CALL lbc_lnk( v_io (:,:), 'I', -1. )   ! I-point (i.e. ice U-V point)

!!gmbug  in the ocean freezing point computed as :
!!gm           fzptn (ji,jj) = ( -0.0575 + 1.710523e-3 * SQRT( sn(ji,jj,1) )   &
!!gm                                     - 2.154996e-4 *       sn(ji,jj,1)   ) * sn(ji,jj,1)   !!   &
!!gm           !!                        - 7.53e-4 * pressure
!!gm
!!gm!bug this is much more accurate and efficient computation
!!gm       **************************************************
!!gm freezing point from unesco:
!!gm     real function tf(s,p)
!!gm   function to compute the freezing point of seawater
!!gm
!!gm   reference: unesco tech. papers in the marine science no. 28. 1978
!!gm   eighth report jpots
!!gm   annex 6 freezing point of seawater f.j. millero pp.29-35.
!!gm
!!gm  units:
!!gm         pressure      p          decibars
!!gm         salinity      s          pss-78
!!gm         temperature   tf         degrees celsius
!!gm         freezing pt.
!!gm************************************************************
!!gm  checkvalue: tf= -2.588567 deg. c for s=40.0, p=500. decibars
!!gm     tf=(-.0575+1.710523e-3*sqrt(abs(s))-2.154996e-4*s)*s-7.53e-4*p
!!gm     return
!!gm     end
!!gm!bug


!!gm      DO jj = 1, jpj
!!gm         DO ji = 1, jpi
!!gm            tfu(ji,jj)  = (  rt0 + ( - 0.0575                              &
!!gm               &                     + 1.710523e-3 * SQRT( sss_io(ji,jj) )   &
!!gm               &                     - 2.154996e-4 *       sss_io(ji,jj)   ) * sss_io(ji,jj)  ) * tms(ji,jj)
!!gm         END DO
!!gm      END DO
!!gm
      zsss_io (:,:) = SQRT( sss_io(:,:) ) 
      zsss2_io(:,:) =  sss_io(:,:) *  sss_io(:,:)
      zsss3_io(:,:) = zsss_io(:,:) * zsss_io(:,:) * zsss_io(:,:)

      DO jj = 1, jpj
         DO ji = 1, jpi
            tfu(ji,jj)  = ABS ( rt0 - 0.0575       *   sss_io(ji,jj)   &
               &                    + 1.710523e-03 * zsss3_io(ji,jj)   &
               &                    - 2.154996e-04 * zsss2_io(ji,jj) ) * tms(ji,jj)
         END DO
      END DO
   
         

         IF( l_ctl .AND. lwp ) THEN         ! print mean trends (used for debugging)
            WRITE(numout,*) 'Ice Forcings '
            WRITE(numout,*) ' qsr_oce  : ', SUM( qsr_oce (:,:) ), ' qsr_ice  : ', SUM( qsr_ice (:,:) )
            WRITE(numout,*) ' qnsr_oce : ', SUM( qnsr_oce(:,:) ), ' qnsr_ice : ', SUM( qnsr_ice(:,:) )
            WRITE(numout,*) ' evap     : ', SUM( evap (:,:) )
            WRITE(numout,*) ' precip   : ', SUM( tprecip(:,:)  ), ' Snow     : ', SUM( sprecip (:,:) )
            WRITE(numout,*) ' u-stress : ', SUM( gtaux  (:,:)  ), ' v-stress : ', SUM( gtauy  (:,:) )
            WRITE(numout,*) ' sst      : ', SUM( sst_io (:,:)  ), ' sss      : ', SUM( sss_io (:,:) )
            WRITE(numout,*) ' u_io     : ', SUM( u_io   (:,:)  ), ' v_io     : ', SUM( v_io   (:,:) )
            WRITE(numout,*) ' hsnif  1 : ', SUM( hsnif  (:,:)  ), ' hicnif   : ', SUM( hicif  (:,:) )
            WRITE(numout,*) ' frld   1 : ', SUM( frld   (:,:)  ), ' sist     : ', SUM( sist   (:,:) )
         ENDIF

         ! Ice model call
         numit = numit + nfice 

         !                                                           !--------------!
         CALL lim_dyn                                                ! Ice dynamics !   ( rheology/dynamics )
         !                                                           !--------------!
         IF( l_ctl .AND. lwp ) THEN
            WRITE(numout,*) ' hsnif  2 : ', SUM( hsnif  (:,:) ), ' hicnif   : ', SUM( hicif  (:,:) )
            WRITE(numout,*) ' frld   2 : ', SUM( frld   (:,:) ), ' sist     : ', SUM( sist   (:,:) )
         ENDIF


         !                                                           !---------------!
         CALL lim_trp                                                ! Ice transport !  ( Advection/diffusion )
         !                                                           !---------------!
         IF( l_ctl .AND. lwp ) THEN
            WRITE(numout,*) ' hsnif  3 : ', SUM( hsnif  (:,:) ), ' hicnif   : ', SUM( hicif  (:,:) )
            WRITE(numout,*) ' frld   3 : ', SUM( frld   (:,:) ), ' sist     : ', SUM( sist   (:,:) )
         ENDIF


         !                                                           !--------------------!
         CALL lim_thd                                                ! Ice thermodynamics !
         !                                                           !--------------------!
         IF( l_ctl .AND. lwp ) THEN
            WRITE(numout,*) ' hsnif  4 : ', SUM( hsnif  (:,:) ), ' hicnif   : ', SUM( hicif  (:,:) )
            WRITE(numout,*) ' frld   4 : ', SUM( frld   (:,:) ), ' sist     : ', SUM( sist   (:,:) )
         ENDIF


         ! Mass and heat fluxes from ice to ocean
         !                                                           !------------------------------!
         CALL lim_flx                                                ! Ice/Ocean Mass & Heat fluxes !
         !                                                           !------------------------------!

         IF( MOD( numit, ninfo ) == 0 .OR. ntmoy == 1 )  THEN        !-----------------!
            CALL lim_dia                                             ! Ice Diagnostics !
         ENDIF                                                       !-----------------!

         !                                                           !-------------!
         CALL lim_wri                                                ! Ice outputs !
         !                                                           !-------------!

         IF( MOD( numit, nstock ) == 0 .OR. numit == nlast ) THEN
            !                                                        !------------------!
            CALL lim_rst_write( numit )                              ! Ice restart file !
            !                                                        !------------------!
         ENDIF

         ! Re-initialization of forcings
         qsr_oce (:,:) = 0.e0
         qsr_ice (:,:) = 0.e0
         qnsr_oce(:,:) = 0.e0
         qnsr_ice(:,:) = 0.e0 
         dqns_ice(:,:) = 0.e0 
         tprecip (:,:) = 0.e0 
         sprecip (:,:) = 0.e0
#if defined key_coupled 
         rrunoff (:,:) = 0.e0
         calving (:,:) = 0.e0
#else
         qla_ice (:,:) = 0.e0
         dqla_ice(:,:) = 0.e0
         fr1_i0  (:,:) = 0.e0
         fr2_i0  (:,:) = 0.e0
         evap    (:,:) = 0.e0
#endif

      ENDIF

   END SUBROUTINE ice_stp

#else
   !!----------------------------------------------------------------------
   !!   Default option                                 NO LIM sea-ice model
   !!----------------------------------------------------------------------
   USE in_out_manager

CONTAINS

   SUBROUTINE ice_stp ( kt )            ! Empty routine
      INTEGER, INTENT( in ) ::   kt     ! ocean time-step index

      IF( kt == nit000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'No Sea Ice Model'
         IF(lwp) WRITE(numout,*) '~~~~~~~'
      ENDIF

   END SUBROUTINE ice_stp

#endif

   !!======================================================================
END MODULE icestp
