MODULE limflx
   !!======================================================================
   !!                       ***  MODULE limflx   ***
   !!           computation of the flux at the sea ice/ocean interface
   !!======================================================================
#if defined key_ice_lim
   !!----------------------------------------------------------------------
   !!   'key_ice_lim'                                     LIM sea-ice model
   !!----------------------------------------------------------------------
   !!   lim_flx  : flux at the ice / ocean interface
   !! * Modules used
   USE phycst
   USE ocfzpt
   USE ice_oce
   USE flx_oce
   USE ice
   USE flxblk
   USE lbclnk
   USE in_out_manager

   IMPLICIT NONE
   PRIVATE

   !! * Routine accessibility
   PUBLIC lim_flx       ! called by lim_step

  !! * Module variables
     REAL(wp)  ::            &  ! constant values
         epsi16 = 1e-16   ,  &
         rzero  = 0.0    ,  &
         rone   = 1.0
   !! * Substitutions
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!   LIM 2.0 , UCL-LODYC-IPSL  (2003)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE lim_flx
      !!-------------------------------------------------------------------
      !!                ***  ROUTINE lim_flx ***
      !!                
      !!  
      !! ** Purpose : Computes the mass and heat fluxes to the ocean
      !!         
      !! ** Action  : - Initialisation of some variables
      !!              - comput. of the fluxes at the sea ice/ocean interface
      !!     
      !! ** Outputs : - fsolar  : solar heat flux at sea ice/ocean interface
      !!              - fnsolar : non solar heat flux 
      !!              - fsalt   : salt flux at sea ice/ocean interface
      !!              - fmass   : freshwater flux at sea ice/ocean interface
      !!
      !!
      !! ** References :
      !!       H. Goosse et al. 1996, Bul. Soc. Roy. Sc. Liege, 65, 87-90
      !!         original    : 00-01 (LIM)
      !!         addition    : 02-07 (C. Ethe, G. Madec)
      !!---------------------------------------------------------------------
      !! * Modules used
      !! * Local variables
      INTEGER ::   ji, jj         ! dummy loop indices

      INTEGER ::   &
         ifvt, i1mfr, idfr ,   &  ! some switches
         iflt, ial, iadv, ifral, ifrdv
      
      REAL(wp) ::   &
         zinda  ,              &  ! switch for testing the values of ice concentration
         z1mthcm,              &  ! 1 - thcm
!!         zfcm1  ,              &  ! solar  heat fluxes
!!         zfcm2  ,              &  !  non solar heat fluxes
#if defined key_lim_fdd   
         zfons,                &  ! salt exchanges at the ice/ocean interface
         zpme                     ! freshwater exchanges at the ice/ocean interface
#else
         zprs  , zfons,        &  ! salt exchanges at the ice/ocean interface
         zpmess                   ! freshwater exchanges at the ice/ocean interface
#endif
      REAL(wp), DIMENSION(jpi,jpj) ::  &
         zfcm1  ,              &  ! solar  heat fluxes
         zfcm2                    !  non solar heat fluxes      
#if defined key_coupled    
      REAL(wp), DIMENSION(jpi,jpj) ::  &
         zalb  ,               &  ! albedo of ice under overcast sky
         zalcn ,               &  ! albedo of ocean under overcast sky
         zalbp ,               &  ! albedo of ice under clear sky
         zaldum                   ! albedo of ocean under clear sky
#endif
      !!---------------------------------------------------------------------
     
      !---------------------------------!
      !      Sea ice/ocean interface    !
      !---------------------------------!
       
       
      !      heat flux at the ocean surface
      !-------------------------------------------------------
       
      DO jj = 1, jpj
         DO ji = 1, jpi
            zinda   = 1.0 - MAX( rzero , SIGN( rone , - ( 1.0 - pfrld(ji,jj) ) ) )
            ifvt    = zinda  *  MAX( rzero , SIGN( rone, -phicif(ji,jj) ) )
            i1mfr   = 1.0 - MAX( rzero , SIGN( rone ,  - ( 1.0 - frld(ji,jj) ) ) )
            idfr    = 1.0 - MAX( rzero , SIGN( rone , frld(ji,jj) - pfrld(ji,jj) ) )
            iflt    = zinda  * (1 - i1mfr) * (1 - ifvt )
            ial     = ifvt   * i1mfr + ( 1 - ifvt ) * idfr
            iadv    = ( 1  - i1mfr ) * zinda
            ifral   = ( 1  - i1mfr * ( 1 - ial ) )   
            ifrdv   = ( 1  - ifral * ( 1 - ial ) ) * iadv 
            z1mthcm =   1. - thcm(ji,jj)       
            !   computation the solar flux at ocean surface
            zfcm1(ji,jj)   = pfrld(ji,jj) * qsr_oce(ji,jj)  + ( 1. - pfrld(ji,jj) ) * fstric(ji,jj)
            !  computation the non solar heat flux at ocean surface
            zfcm2(ji,jj) =  - z1mthcm * zfcm1(ji,jj)   &
               &           + iflt    * ( fscmbq(ji,jj) + ffltbif(ji,jj) )                            &
               &           + ifral   * ( ial * qcmif(ji,jj) + (1 - ial) * qldif(ji,jj) ) / rdt_ice   &
               &           + ifrdv   * ( qfvbq(ji,jj) + qdtcn(ji,jj) ) / rdt_ice

            fsbbq(ji,jj) = ( 1.0 - ( ifvt + iflt ) ) * fscmbq(ji,jj)     ! ???
            
            fsolar (ji,jj) = zfcm1(ji,jj)                                       ! solar heat flux 

            fnsolar(ji,jj) = zfcm2(ji,jj) - fdtcn(ji,jj)                        ! non solar heat flux
         END DO
      END DO
  
       
      !      mass flux at the ocean surface
      !-------------------------------------------------------
       
      DO jj = 1, jpj
         DO ji = 1, jpi
#if defined key_lim_fdd
            !  case of realistic freshwater flux (Tartinville et al., 2001)
            
            !  computing freshwater exchanges at the ice/ocean interface
            zpme = - evap(ji,jj) * frld(ji,jj)            &   !  evaporation over oceanic fraction
               &   + tprecip(ji,jj)                            &   !  total precipitation
               &   - sprecip(ji,jj) * ( 1. - pfrld(ji,jj) )  &   !  remov. snow precip over ice
               &   - rdmsnif(ji,jj) / rdt_ice                   !  freshwaterflux due to snow melting 
            
            !  computing salt exchanges at the ice/ocean interface
            zfons =  ( soce - sice ) * ( rdmicif(ji,jj) / rdt_ice ) 
            
            !  converting the salt flux from ice to a freshwater flux from ocean
            fsalt(ji,jj) = zfons / ( sss_io(ji,jj) + epsi16 )
            
            !  freshwater masses
            fmass(ji,jj) = - zpme 
#else
            !  case of freshwater flux equivalent as salt flux
            !  dilution effect due to evaporation and precipitation
            zprs  = ( tprecip(ji,jj) - sprecip(ji,jj) * ( 1. - pfrld(ji,jj) ) ) * soce  
            !  freshwater flux
            zfons = rdmicif(ji,jj) * ( soce - sice )  &  !  fwf : ice formation and melting
               &   -  dmgwi(ji,jj) * sice             &  !  fwf : salt flx needed to bring the fresh snow to sea/ice salinity
               &   + rdmsnif(ji,jj) * soce               !  fwf to ocean due to snow melting 
            !  salt exchanges at the ice/ocean interface
            zpmess         =  zprs - zfons / rdt_ice - evap(ji,jj) * soce * frld(ji,jj)
            fsalt(ji,jj) =  - zpmess
#endif
         END DO
      END DO


      !-------------------------------------------------------------------!
      !  computation of others transmitting variables from ice to ocean   !
      !------------------------------------------ ------------------------!

      !-----------------------------------------------!
      !   Storing the transmitted variables           !
      !-----------------------------------------------!

      DO jj = 1, jpj
         DO ji = 1, jpi
            ftaux (ji,jj) = - tio_u(ji,jj) * rau0   ! taux ( ice: N/m2/rau0, ocean: N/m2 )
            ftauy (ji,jj) = - tio_v(ji,jj) * rau0   ! tauy ( ice: N/m2/rau0, ocean: N/m2 )                
            freeze(ji,jj) = 1.0 - frld(ji,jj)       ! Sea ice cover            
            tn_ice(ji,jj) = sist(ji,jj)             ! Ice surface temperature                      
         END DO
      END DO

#if defined key_coupled            
      zalb  (:,:) = 0.e0
      zalcn (:,:) = 0.e0
      zalbp (:,:) = 0.e0
      zaldum(:,:) = 0.e0

      !------------------------------------------------!
      !  2) Computation of snow/ice and ocean albedo   !
      !------------------------------------------------!
      CALL flx_blk_albedo( zalb, zalcn, zalbp, zaldum )

      DO jj = 1, jpj
         DO ji = 1, jpi
            alb_ice(ji,jj) =  0.5 * zalbp(ji,jj) + 0.5 * zalb (ji,jj)  ! Ice albedo                       
         END DO
      END DO
#endif

      IF( l_ctl .AND. lwp ) THEN
         WRITE(numout,*) ' lim_flx  '
         WRITE(numout,*) ' fsolar ', SUM(fsolar), ' fnsolar', SUM( fnsolar )
         WRITE(numout,*) ' fmass  ', SUM(fmass ), ' fsalt  ', SUM(fsalt)
         WRITE(numout,*) ' ftaux  ', SUM(ftaux ), ' ftauy  ', SUM(ftauy)
         WRITE(numout,*) ' freeze ', SUM(freeze), ' tn_ice ', SUM(tn_ice)
      ENDIF 
 
   
    END SUBROUTINE lim_flx

#else
   !!----------------------------------------------------------------------
   !!   Default option :        Empty module           NO LIM sea-ice model
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE lim_flx         ! Empty routine
   END SUBROUTINE lim_flx
#endif 

END MODULE limflx
