MODULE diafwb
   !!======================================================================
   !!                       ***  MODULE  diafwb  ***
   !! Ocean diagnostics: freshwater budget
   !!======================================================================
#if ( defined key_orca_r2 || defined  key_orca_r4 ) && defined key_dynspg_fsc && ! defined key_coupled
   !!----------------------------------------------------------------------
   !!   "key_dynspg_fsc" and "key_orca_r2 or 4"
   !!----------------------------------------------------------------------
   !!   dia_fwb     : freshwater budget for global ocean configurations
   !!----------------------------------------------------------------------
   !! * Modules used
   USE oce             ! ocean dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE phycst          ! physical constants
   USE zdf_oce         ! ocean vertical physics
   USE in_out_manager  ! I/O manager
   USE flxrnf          ! ???
   USE ocesbc          ! ???
   USE blk_oce         ! ???
   USE flxblk          ! atmospheric surface quantity
   USE lib_mpp         ! distributed memory computing library

   IMPLICIT NONE
   PRIVATE

   !! * Routine accessibility
   PUBLIC dia_fwb    ! routine called by step.F90

   !! * Shared module variables
   LOGICAL, PUBLIC, PARAMETER ::   lk_diafwb = .TRUE.    ! fresh water budget flag

   !! * Module variables
   REAL(wp) ::   &
      a_emp, a_precip, a_rnf,   &
      a_sshb, a_sshn, a_salb, a_saln,   &
      a_aminus, a_aplus
   REAL(wp), DIMENSION(4) ::   &
      a_flxi, a_flxo, a_temi, a_temo, a_sali, a_salo

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LODYC-IPSL  (2003)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE dia_fwb( kt )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE dia_fwb  ***
      !!     
      !! ** Purpose :
      !!	
      !! ** Method :
      !!	
      !! History :
      !!   8.2  !  01-02  (E. Durand)  Original code
      !!   8.5  !  02-06  (G. Madec)  F90: Free form and module
      !!----------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( in ) ::   kt      ! ocean time-step index

      !! * Local declarations
      INTEGER :: ji, jj, jk, jt   ! dummy loop indices
      REAL(wp) ::   zarea, zvol, zwei
      REAL(wp) ::  ztemi(4), ztemo(4), zsali(4), zsalo(4), zflxi(4), zflxo(4)
      REAL(wp) ::  zt, zs, zu  
      REAL(wp) ::  zsm0, zempnew
      !!----------------------------------------------------------------------

      ! Mean global salinity
      zsm0 = 34.72654

      ! To compute emp mean value mean emp

      IF( kt == nit000 ) THEN

         a_emp    = 0.e0
         a_precip = 0.e0
         a_rnf    = 0.e0
         a_sshb   = 0.e0 ! valeur de ssh au debut de la simulation
         a_salb   = 0.e0 ! valeur de sal au debut de la simulation
         a_aminus = 0.e0
         a_aplus  = 0.e0
         ! sshb used because diafwb called after tranxt (i.e. after the swap)
         a_sshb = SUM( e1t(:,:) * e2t(:,:) * sshb(:,:) * tmask_i(:,:) )

         DO jk = 1, jpkm1
            DO jj = 2, jpjm1
               DO ji = fs_2, fs_jpim1   ! vector opt.
                  zwei  = e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk) * tmask(ji,jj,jk) * tmask_i(ji,jj)
                  a_salb = a_salb + ( sb(ji,jj,jk) - zsm0 ) * zwei
               END DO
            END DO
         END DO
      ENDIF
      
      a_emp    = SUM( e1t(:,:) * e2t(:,:) * emp   (:,:) * tmask_i(:,:) )
#if defined key_flx_bulk_monthly || defined key_flx_bulk_daily
      a_precip = SUM( e1t(:,:) * e2t(:,:) * watm  (:,:) * tmask_i(:,:) )
#endif
      a_rnf    = SUM( e1t(:,:) * e2t(:,:) * runoff(:,:) * tmask_i(:,:) )

      IF( aminus /= 0.0 ) a_aminus = a_aminus + ( MIN( aplus, aminus ) / aminus )
      IF( aplus  /= 0.0 ) a_aplus  = a_aplus  + ( MIN( aplus, aminus ) / aplus  )

#if defined key_mpp
      ! Mpp: sum over all the global domain
      CALL  mpp_sum( a_sshn )                            !!!!!! bugggggg   a_sshn note befined before!!!!!
#endif

      IF( kt == nitend ) THEN
         a_sshn = 0.e0
         a_saln = 0.e0
         zarea = 0.e0
         zvol  = 0.e0
         zempnew = 0.e0
         ! Mean sea level at nitend
         a_sshn = SUM( e1t(:,:) * e2t(:,:) * sshn(:,:) * tmask_i(:,:) )
         zarea  = SUM( e1t(:,:) * e2t(:,:) *             tmask_i(:,:) )
         
         DO jk = 1, jpkm1   
            DO jj = 2, jpjm1
               DO ji = fs_2, fs_jpim1   ! vector opt.
                  zwei  = e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk) * tmask(ji,jj,jk) * tmask_i(ji,jj)
                  a_saln = a_saln + ( sn(ji,jj,jk) - zsm0 ) * zwei
                  zvol  = zvol  + zwei
               END DO
            END DO
         END DO
         
         a_aminus = a_aminus/(nitend-nit000+1)
         a_aplus  = a_aplus/(nitend-nit000+1)

         ! Conversion in m3
         a_emp    = a_emp * rdttra(1) * 1.e-3 
         a_precip = a_precip * rdttra(1) * 1.e-3 / rday
         a_rnf    = a_rnf * rdttra(1) * 1.e-3
         
         ! Alpha1=Alpha0-Rest/(Precip+runoff)
         !  C A U T I O N : precipitations are negative !!     
         
         zempnew = a_sshn / ( ( nitend - nit000 + 1 ) * rdt ) * 1.e3 / zarea

      ENDIF


      ! Calcul des termes de transport 
      ! ------------------------------
      
      ! 1 --> Gibraltar 
      ! 2 --> Cadiz 
      ! 3 --> Red Sea
      ! 4 --> Baltic Sea

      IF( kt == nit000 ) THEN
         a_flxi(:) = 0.e0
         a_flxo(:) = 0.e0
         a_temi(:) = 0.e0
         a_temo(:) = 0.e0
         a_sali(:) = 0.e0
         a_salo(:) = 0.e0
      ENDIF

      zflxi(:) = 0.e0
      zflxo(:) = 0.e0
      ztemi(:) = 0.e0
      ztemo(:) = 0.e0
      zsali(:) = 0.e0
      zsalo(:) = 0.e0

      ! Mean flow at Gibraltar

      IF( cp_cfg == "orca" ) THEN  
               
         SELECT CASE ( jp_cfg )
         !                                           ! =======================
         CASE ( 4 )                                  !  ORCA_R4 configuration
            !                                        ! =======================
            ji =  mi1(70)
            jj =  mj1(52)
            !                                        ! =======================
         CASE ( 2 )                                  !  ORCA_R2 configuration
            !                                        ! =======================
            ji = mi1(139)
            jj = mj1(102)
            !                                        ! =======================
         CASE DEFAULT                                !    ORCA R05 or R025
            !                                        ! =======================
            IF(lwp) WRITE(numout,cform_err)
            IF(lwp) WRITE(numout,*)' dia_fwb Not yet implemented in ORCA_R05 or R025'
            nstop = nstop + 1
            ! 
         END SELECT
         ! 
      ENDIF
      DO jk = 1, 18 
         zt = 0.5 * ( tn(ji,jj,jk) + tn(ji+1,jj,jk) )
         zs = 0.5 * ( sn(ji,jj,jk) + sn(ji+1,jj,jk) )
         zu = un(ji,jj,jk) * fse3t(ji,jj,jk) * e2u(ji,jj)

         IF( un(ji,jj,jk) > 0.0 ) THEN 
            zflxi(1) = zflxi(1) +    zu
            ztemi(1) = ztemi(1) + zt*zu
            zsali(1) = zsali(1) + zs*zu
         ELSE
            zflxo(1) = zflxo(1) +    zu
            ztemo(1) = ztemo(1) + zt*zu
            zsalo(1) = zsalo(1) + zs*zu
         ENDIF
      END DO
      
      ! Mean flow at Cadiz
      IF( cp_cfg == "orca" ) THEN
               
         SELECT CASE ( jp_cfg )
         !                                           ! =======================
         CASE ( 4 )                                  !  ORCA_R4 configuration
            !                                        ! =======================
            ji =  mi1(69  )
            jj =  mj1(52  )
            !                                        ! =======================
         CASE ( 2 )                                  !  ORCA_R2 configuration
            !                                        ! =======================
            ji = mi1(137)
            jj = mj1(102)
            !                                        ! =======================
         CASE DEFAULT                                !    ORCA R05 or R025
            !                                        ! =======================
            IF(lwp) WRITE(numout,cform_err)
            IF(lwp) WRITE(numout,*)' dia_fwb Not yet implemented in ORCA_R05 or R025'
            nstop = nstop + 1
            ! 
         END SELECT
         ! 
      ENDIF
      DO jk = 1, 23 
         zt = 0.5 * ( tn(ji,jj,jk) + tn(ji+1,jj,jk) )
         zs = 0.5 * ( sn(ji,jj,jk) + sn(ji+1,jj,jk) )
         zu = un(ji,jj,jk) * fse3t(ji,jj,jk) * e2u(ji,jj)
         
         IF( un(ji,jj,jk) > 0.0 ) THEN 
            zflxi(2) = zflxi(2) +    zu
            ztemi(2) = ztemi(2) + zt*zu
            zsali(2) = zsali(2) + zs*zu
         ELSE
            zflxo(2) = zflxo(2) +    zu
            ztemo(2) = ztemo(2) + zt*zu
            zsalo(2) = zsalo(2) + zs*zu
         ENDIF
      END DO

      ! Mean flow at Red Sea entrance
      IF( cp_cfg == "orca" ) THEN
               
         SELECT CASE ( jp_cfg )
         !                                           ! =======================
         CASE ( 4 )                                  !  ORCA_R4 configuration
            !                                        ! =======================
            ji =  mi1(83  )
            jj =  mj1(45  )
            !                                        ! =======================
         CASE ( 2 )                                  !  ORCA_R2 configuration
            !                                        ! =======================
            ji = mi1(161)
            jj = mj1( 88)
            !                                        ! =======================
         CASE DEFAULT                                !    ORCA R05 or R025
            !                                        ! =======================
            IF(lwp) WRITE(numout,cform_err)
            IF(lwp) WRITE(numout,*)' dia_fwb Not yet implemented in ORCA_R05 or R025'
            nstop = nstop + 1
            ! 
         END SELECT
         ! 
      ENDIF
      DO jk = 1, 15 
         zt = 0.5 * ( tn(ji,jj,jk) + tn(ji+1,jj,jk) )
         zs = 0.5 * ( sn(ji,jj,jk) + sn(ji+1,jj,jk) )
         zu = un(ji,jj,jk) * fse3t(ji,jj,jk) * e2u(ji,jj)
         
         IF( un(ji,jj,jk) > 0.0 ) THEN 
            zflxi(3) = zflxi(3) +    zu
            ztemi(3) = ztemi(3) + zt*zu
            zsali(3) = zsali(3) + zs*zu
         ELSE
            zflxo(3) = zflxo(3) +    zu
            ztemo(3) = ztemo(3) + zt*zu
            zsalo(3) = zsalo(3) + zs*zu
         ENDIF
      END DO

      ! Mean flow at Baltic Sea entrance
      IF( cp_cfg == "orca" ) THEN
               
         SELECT CASE ( jp_cfg )
         !                                           ! =======================
         CASE ( 4 )                                  !  ORCA_R4 configuration
            !                                        ! =======================
            ji =   1     ! Not in the domain
            jj =   1  
            !                                        ! =======================
         CASE ( 2 )                                  !  ORCA_R2 configuration
            !                                        ! =======================
            ji = mi1(146)
            jj = mj1(116)
            !                                        ! =======================
         CASE DEFAULT                                !    ORCA R05 or R025
            !                                        ! =======================
            IF(lwp) WRITE(numout,cform_err)
            IF(lwp) WRITE(numout,*)' dia_fwb Not yet implemented in ORCA_R05 or R025'
            nstop = nstop + 1
            ! 
         END SELECT
         ! 
      ENDIF
      DO jk = 1, 20
         zt = 0.5 * ( tn(ji,jj,jk) + tn(ji+1,jj,jk) )
         zs = 0.5 * ( sn(ji,jj,jk) + sn(ji+1,jj,jk) )
         zu = un(ji,jj,jk) * fse3t(ji,jj,jk) * e2u(ji,jj)
         
         IF( un(ji,jj,jk) > 0.0 ) THEN 
            zflxi(4) = zflxi(4) +    zu
            ztemi(4) = ztemi(4) + zt*zu
            zsali(4) = zsali(4) + zs*zu
         ELSE
            zflxo(4) = zflxo(4) +    zu
            ztemo(4) = ztemo(4) + zt*zu
            zsalo(4) = zsalo(4) + zs*zu
         ENDIF
      END DO

      ! Sum at each time-step
      DO jt = 1, 4 
         IF( zflxi(jt) /= 0.0 .AND. zflxo(jt) /= 0.0 ) THEN
            a_flxi(jt) = a_flxi(jt) + zflxi(jt)
            a_temi(jt) = a_temi(jt) + ztemi(jt)/zflxi(jt)
            a_sali(jt) = a_sali(jt) + zsali(jt)/zflxi(jt)
            a_flxo(jt) = a_flxo(jt) + zflxo(jt)
            a_temo(jt) = a_temo(jt) + ztemo(jt)/zflxo(jt)
            a_salo(jt) = a_salo(jt) + zsalo(jt)/zflxo(jt)
         ENDIF
      END DO

      IF( kt == nitend ) THEN
         DO jt = 1, 4 
            a_flxi(jt) = a_flxi(jt)/((nitend-nit000+1)*1.e6)
            a_temi(jt) = a_temi(jt)/( nitend-nit000+1)
            a_sali(jt) = a_sali(jt)/( nitend-nit000+1)
            a_flxo(jt) = a_flxo(jt)/((nitend-nit000+1)*1.e6)
            a_temo(jt) = a_temo(jt)/( nitend-nit000+1)
            a_salo(jt) = a_salo(jt)/( nitend-nit000+1)
         END DO
      ENDIF


      ! Ecriture des diagnostiques 
      ! --------------------------

      IF ( kt == nitend ) THEN

         OPEN(111,FILE='STRAIT.dat')
         WRITE(111,*)
         WRITE(111,*)    'Net freshwater budget '
         WRITE(111,9010) '  emp    = ',a_emp,   ' m3 =', a_emp   /((nitend-nit000+1)*rdttra(1)) * 1.e-6,' Sv'
         WRITE(111,9010) '  precip = ',a_precip,' m3 =', a_precip/((nitend-nit000+1)*rdttra(1)) * 1.e-6,' Sv'
         WRITE(111,9010) '  a_rnf   = ',a_rnf,   ' m3 =', a_rnf   /((nitend-nit000+1)*rdttra(1)) * 1.e-6,' Sv'
         WRITE(111,*)
         WRITE(111,9010) '  zarea =',zarea
         WRITE(111,9010) '  zvol  =',zvol
         WRITE(111,*)
         WRITE(111,*)    'Mean sea level : '
         WRITE(111,9010) '  at nit000 = ',a_sshb        ,' m3 '
         WRITE(111,9010) '  at nitend = ',a_sshn        ,' m3 '
         WRITE(111,9010) '  diff      = ',(a_sshn-a_sshb),' m3 =', (a_sshn-a_sshb)/((nitend-nit000+1)*rdt) * 1.e-6,' Sv'
         WRITE(111,9020) '  mean sea level elevation    =', a_sshn/zarea,' m'
         WRITE(111,*)
         WRITE(111,*)    'Anomaly of salinity content : '
         WRITE(111,9010) '  at nit000 = ',a_salb        ,' psu.m3 '
         WRITE(111,9010) '  at nitend = ',a_saln        ,' psu.m3 '
         WRITE(111,9010) '  diff      = ',(a_saln-a_salb),' psu.m3'
         WRITE(111,*)
         WRITE(111,*)    'Mean salinity : '
         WRITE(111,9020) '  at nit000 =',a_salb/zvol+zsm0   ,' psu '
         WRITE(111,9020) '  at nitend =',a_saln/zvol+zsm0   ,' psu '
         WRITE(111,9020) '  diff      =',(a_saln-a_salb)/zvol,' psu'
         WRITE(111,9020) '  S-SLevitus=',a_saln/zvol,' psu'
         WRITE(111,*)
         WRITE(111,*)    'Coeff : '
         WRITE(111,9030) '  Alpha+   =  ', a_aplus
         WRITE(111,9030) '  Alpha-   =  ', a_aminus
         WRITE(111,*)
         WRITE(111,*)
         WRITE(111,*)    'Gibraltar : '
         WRITE(111,9030) '  Flux entrant (Sv) :', a_flxi(1)
         WRITE(111,9030) '  Flux sortant (Sv) :', a_flxo(1)
         WRITE(111,9030) '  T entrant (deg)   :', a_temi(1)
         WRITE(111,9030) '  T sortant (deg)   :', a_temo(1)
         WRITE(111,9030) '  S entrant (psu)   :', a_sali(1)
         WRITE(111,9030) '  S sortant (psu)   :', a_salo(1)
         WRITE(111,*)
         WRITE(111,*)    'Cadiz : '
         WRITE(111,9030) '  Flux entrant (Sv) :', a_flxi(2)
         WRITE(111,9030) '  Flux sortant (Sv) :', a_flxo(2)
         WRITE(111,9030) '  T entrant (deg)   :', a_temi(2)
         WRITE(111,9030) '  T sortant (deg)   :', a_temo(2)
         WRITE(111,9030) '  S entrant (psu)   :', a_sali(2)
         WRITE(111,9030) '  S sortant (psu)   :', a_salo(2)
         WRITE(111,*)
         WRITE(111,*)    'Bab el Mandeb : '
         WRITE(111,9030) '  Flux entrant (Sv) :', a_flxi(3)
         WRITE(111,9030) '  Flux sortant (Sv) :', a_flxo(3)
         WRITE(111,9030) '  T entrant (deg)   :', a_temi(3)
         WRITE(111,9030) '  T sortant (deg)   :', a_temo(3)
         WRITE(111,9030) '  S entrant (psu)   :', a_sali(3)
         WRITE(111,9030) '  S sortant (psu)   :', a_salo(3)
         WRITE(111,*)
         WRITE(111,*)    'Baltic : '
         WRITE(111,9030) '  Flux entrant (Sv) :', a_flxi(4)
         WRITE(111,9030) '  Flux sortant (Sv) :', a_flxo(4)
         WRITE(111,9030) '  T entrant (deg)   :', a_temi(4)
         WRITE(111,9030) '  T sortant (deg)   :', a_temo(4)
         WRITE(111,9030) '  S entrant (psu)   :', a_sali(4)
         WRITE(111,9030) '  S sortant (psu)   :', a_salo(4)
         CLOSE(111)
      ENDIF

 9005 FORMAT(1X,A,ES24.16)
 9010 FORMAT(1X,A,ES12.5,A,F10.5,A)
 9020 FORMAT(1X,A,F10.5,A)
 9030 FORMAT(1X,A,F8.2,A)

   END SUBROUTINE dia_fwb

#else
   !!----------------------------------------------------------------------
   !!   Default option :                                       Empty Module
   !!----------------------------------------------------------------------
   LOGICAL, PUBLIC, PARAMETER ::   lk_diafwb = .FALSE.    ! fresh water budget flag
CONTAINS
   SUBROUTINE dia_fwb( kt )        ! Empty routine
      WRITE(*,*) kt                      ! no warning in compilation phase
   END SUBROUTINE dia_fwb
#endif

   !!======================================================================
END MODULE diafwb
