MODULE traadv_tvd
   !!==============================================================================
   !!                       ***  MODULE  traadv_tvd  ***
   !! Ocean active tracers:  horizontal & vertical advective trend
   !!==============================================================================

   !!----------------------------------------------------------------------
   !!   tra_adv_tvd  : update the tracer trend with the horizontal
   !!                  and vertical advection trends using a TVD scheme
   !!   nonosc       : compute monotonic tracer fluxes by a nonoscillatory
   !!                  algorithm 
   !!----------------------------------------------------------------------
   !! * Modules used
   USE oce             ! ocean dynamics and active tracers
   USE dom_oce         ! ocean space and time domain
   USE trdtra_oce     ! ocean active tracer trends
   USE in_out_manager  ! I/O manager
   USE dynspg_fsc      ! surface pressure gradient 
   USE dynspg_fsc_atsk ! autotasked surface pressure gradient
   USE trabbl          ! Advective term of BBL
   USE lib_mpp
   USE lbclnk

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC tra_adv_tvd    ! routine called by step.F90

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LODYC-IPSL (2003)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE tra_adv_tvd( kt )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE tra_adv_tvd  ***
      !! 
      !! **  Purpose :   Compute the now trend due to total advection of 
      !!       tracers and add it to the general trend of tracer equations
      !!
      !! **  Method  :   TVD scheme, i.e. 2nd order centered scheme with
      !!       corrected flux (monotonic correction)
      !!       note: - this advection scheme needs a leap-frog time scheme
      !!
      !! ** Action : - update (ta,sa) with the now advective tracer trends
      !!             - save the trends in (ttrdh,strdh) ('key_trdtra')
      !!
      !! History :
      !!        !  95-12  (L. Mortier)  Original code
      !!        !  00-01  (H. Loukos)  adapted to ORCA 
      !!        !  00-10  (MA Foujols E.Kestenare)  include file not routine
      !!        !  00-12  (E. Kestenare M. Levy)  fix bug in trtrd indexes
      !!        !  01-07  (E. Durand G. Madec)  adaptation to ORCA config
      !!   8.5  !  02-06  (G. Madec)  F90: Free form and module
      !!   9.0  !  04-01  (A. de Miranda, G. Madec, J.M. Molines ): advective bbl
      !!----------------------------------------------------------------------
      !! * Modules used
#if defined key_trabbl_adv
      USE oce                , zun => ua,  &  ! use ua as workspace
         &                     zvn => va      ! use va as workspace
      REAL(wp), DIMENSION(jpi,jpj,jpk) :: zwn
#else
      USE oce                , zun => un,  &  ! When no bbl, zun == un
                               zvn => vn,  &  !             zvn == vn
                               zwn => wn      !             zwn == wn
#endif
      !! * Arguments
      INTEGER, INTENT( in ) ::   kt         ! ocean time-step

      !! * Local declarations
      INTEGER  ::   ji, jj, jk              ! dummy loop indices
      REAL(wp) ::   zta, zsa                ! temporary scalar
      REAL(wp), DIMENSION (jpi,jpj,jpk) ::   &
         zti, ztu, ztv, ztw,             &  ! temporary workspace
         zsi, zsu, zsv, zsw                 !    "           "
      REAL(wp) ::   &
         z2dtt, zbtr, zeu, zev, zew, z2, &  ! temporary scalar
         zfp_ui, zfp_vj, zfp_wk,         &  !    "         "
         zfm_ui, zfm_vj, zfm_wk             !    "         "
      !!----------------------------------------------------------------------

      IF( kt == nit000 .AND. lwp ) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'tra_adv_tvd : TVD advection scheme'
         WRITE(numout,*) '~~~~~~~~~~~'
      ENDIF

      IF( neuler == 0 .AND. kt == nit000 ) THEN
         z2=1.
      ELSE
         z2=2.
      ENDIF

#if defined key_trabbl_adv

      ! Advective Bottom boundary layer: add the velocity
      ! -------------------------------------------------
      zun(:,:,:) = un (:,:,:) - u_bbl(:,:,:)
      zvn(:,:,:) = vn (:,:,:) - v_bbl(:,:,:)
      zwn(:,:,:) = wn (:,:,:) + w_bbl(:,:,:)
#endif

      ! 1. Bottom value : flux set to zero
      ! ---------------
      ztu(:,:,jpk) = 0.e0   ;   zsu(:,:,jpk) = 0.e0
      ztv(:,:,jpk) = 0.e0   ;   zsv(:,:,jpk) = 0.e0
      ztw(:,:,jpk) = 0.e0   ;   zsw(:,:,jpk) = 0.e0
      zti(:,:,jpk) = 0.e0   ;   zsi(:,:,jpk) = 0.e0


      ! 2. upstream advection with initial mass fluxes & intermediate update
      ! --------------------------------------------------------------------
      ! upstream tracer flux in the i and j direction
      DO jk = 1, jpkm1
         DO jj = 1, jpjm1
            DO ji = 1, fs_jpim1   ! vector opt.
               zeu = 0.5 * e2u(ji,jj) * fse3u(ji,jj,jk) * zun(ji,jj,jk)
               zev = 0.5 * e1v(ji,jj) * fse3v(ji,jj,jk) * zvn(ji,jj,jk)
               ! upstream scheme
               zfp_ui = zeu + ABS( zeu )
               zfm_ui = zeu - ABS( zeu )
               zfp_vj = zev + ABS( zev )
               zfm_vj = zev - ABS( zev )
               ztu(ji,jj,jk) = zfp_ui * tb(ji,jj,jk) + zfm_ui * tb(ji+1,jj  ,jk)
               ztv(ji,jj,jk) = zfp_vj * tb(ji,jj,jk) + zfm_vj * tb(ji  ,jj+1,jk)
               zsu(ji,jj,jk) = zfp_ui * sb(ji,jj,jk) + zfm_ui * sb(ji+1,jj  ,jk)
               zsv(ji,jj,jk) = zfp_vj * sb(ji,jj,jk) + zfm_vj * sb(ji  ,jj+1,jk)
            END DO
         END DO
      END DO

      ! upstream tracer flux in the k direction
      ! Surface value
      IF( lk_dynspg_fsc .OR. lk_dynspg_fsc_tsk ) THEN   ! free surface-constant volume
         DO jj = 1, jpj
            DO ji = 1, jpi
               zew = e1t(ji,jj) * e2t(ji,jj) * zwn(ji,jj,1)
               ztw(ji,jj,1) = zew * tb(ji,jj,1)
               zsw(ji,jj,1) = zew * sb(ji,jj,1)
            END DO
         END DO
      ELSE                                      ! rigid lid : flux set to zero
         ztw(:,:,1) = 0.e0
         zsw(:,:,1) = 0.e0
      ENDIF

      ! Interior value
      DO jk = 2, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi
               zew = 0.5 * e1t(ji,jj) * e2t(ji,jj) * zwn(ji,jj,jk)
               zfp_wk = zew + ABS( zew )
               zfm_wk = zew - ABS( zew )
               ztw(ji,jj,jk) = zfp_wk * tb(ji,jj,jk) + zfm_wk * tb(ji,jj,jk-1)
               zsw(ji,jj,jk) = zfp_wk * sb(ji,jj,jk) + zfm_wk * sb(ji,jj,jk-1)
            END DO
         END DO
      END DO

      ! total advective trend
      DO jk = 1, jpkm1
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zbtr = 1./ ( e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk) )
               zti(ji,jj,jk) = - ( ztu(ji,jj,jk) - ztu(ji-1,jj  ,jk  )   &
                  &              + ztv(ji,jj,jk) - ztv(ji  ,jj-1,jk  )   &
                  &              + ztw(ji,jj,jk) - ztw(ji  ,jj  ,jk+1) ) * zbtr
               zsi(ji,jj,jk) = - ( zsu(ji,jj,jk) - zsu(ji-1,jj  ,jk  )   &
                  &              + zsv(ji,jj,jk) - zsv(ji  ,jj-1,jk  )   &
                  &              + zsw(ji,jj,jk) - zsw(ji  ,jj  ,jk+1) ) * zbtr
            END DO
         END DO
      END DO
      

      ! update and guess with monotonic sheme
      DO jk = 1, jpkm1
         z2dtt = z2 * rdttra(jk)
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               ta(ji,jj,jk) =  ta(ji,jj,jk) + zti(ji,jj,jk)
               sa(ji,jj,jk) =  sa(ji,jj,jk) + zsi(ji,jj,jk)
               zti (ji,jj,jk) = ( tb(ji,jj,jk) + z2dtt * zti(ji,jj,jk) ) * tmask(ji,jj,jk)
               zsi (ji,jj,jk) = ( sb(ji,jj,jk) + z2dtt * zsi(ji,jj,jk) ) * tmask(ji,jj,jk)
            END DO
         END DO
      END DO

      ! Lateral boundary conditions on zti, zsi   (unchanged sign)
      CALL lbc_lnk( zti, 'T', 1. )
      CALL lbc_lnk( zsi, 'T', 1. )


      ! 3. antidiffusive flux : high order minus low order
      ! --------------------------------------------------
      ! antidiffusive flux on i and j
      DO jk = 1, jpkm1
         DO jj = 1, jpjm1
            DO ji = 1, fs_jpim1   ! vector opt.
               zeu = 0.5 * e2u(ji,jj) * fse3u(ji,jj,jk) * zun(ji,jj,jk)
               zev = 0.5 * e1v(ji,jj) * fse3v(ji,jj,jk) * zvn(ji,jj,jk)
               ztu(ji,jj,jk) = zeu * ( tn(ji,jj,jk) + tn(ji+1,jj,jk) ) - ztu(ji,jj,jk)
               zsu(ji,jj,jk) = zeu * ( sn(ji,jj,jk) + sn(ji+1,jj,jk) ) - zsu(ji,jj,jk)
               ztv(ji,jj,jk) = zev * ( tn(ji,jj,jk) + tn(ji,jj+1,jk) ) - ztv(ji,jj,jk)
               zsv(ji,jj,jk) = zev * ( sn(ji,jj,jk) + sn(ji,jj+1,jk) ) - zsv(ji,jj,jk)
            END DO
         END DO
      END DO
      
      ! antidiffusive flux on k
      ! Surface value
      ztw(:,:,1) = 0.
      zsw(:,:,1) = 0.

      ! Interior value
      DO jk = 2, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi
               zew = 0.5 * e1t(ji,jj) * e2t(ji,jj) * zwn(ji,jj,jk)
               ztw(ji,jj,jk) = zew * ( tn(ji,jj,jk) + tn(ji,jj,jk-1) ) - ztw(ji,jj,jk)
               zsw(ji,jj,jk) = zew * ( sn(ji,jj,jk) + sn(ji,jj,jk-1) ) - zsw(ji,jj,jk)
            END DO
         END DO
      END DO

      ! Lateral bondary conditions
      CALL lbc_lnk( ztu, 'U', -1. )   ;   CALL lbc_lnk( zsu, 'U', -1. )
      CALL lbc_lnk( ztv, 'V', -1. )   ;   CALL lbc_lnk( zsv, 'V', -1. )
      CALL lbc_lnk( ztw, 'W',  1. )   ;   CALL lbc_lnk( zsw, 'W',  1. )

      ! 4. monotonicity algorithm
      ! -------------------------
      CALL nonosc( tb, ztu, ztv, ztw, zti, z2 )
      CALL nonosc( sb, zsu, zsv, zsw, zsi, z2 )


      ! 5. final trend with corrected fluxes
      ! ------------------------------------
      DO jk = 1, jpkm1
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zbtr = 1. / ( e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk) )
               ta(ji,jj,jk) = ta(ji,jj,jk)   &
                  &         - ( ztu(ji,jj,jk) - ztu(ji-1,jj  ,jk  )   &
                  &           + ztv(ji,jj,jk) - ztv(ji  ,jj-1,jk  )   &
                  &           + ztw(ji,jj,jk) - ztw(ji  ,jj  ,jk+1) ) * zbtr
               sa(ji,jj,jk) = sa(ji,jj,jk)   &
                  &         - ( zsu(ji,jj,jk) - zsu(ji-1,jj  ,jk  )   &
                  &           + zsv(ji,jj,jk) - zsv(ji  ,jj-1,jk  )   &
                  &           + zsw(ji,jj,jk) - zsw(ji  ,jj  ,jk+1) ) * zbtr
            END DO
         END DO
      END DO

      IF( l_ctl .AND. lwp ) THEN
         zta = SUM( ta(2:jpim1,2:jpjm1,1:jpkm1) * tmask(2:jpim1,2:jpjm1,1:jpkm1) )
         zsa = SUM( sa(2:jpim1,2:jpjm1,1:jpkm1) * tmask(2:jpim1,2:jpjm1,1:jpkm1) )
         WRITE(numout,*) ' zad  - Ta: ', zta-t_ctl, ' Sa: ', zsa-s_ctl, ' tvd'
         t_ctl = zta   ;   s_ctl = zsa
      ENDIF

#if defined key_diaptr
      IF( MOD( kt, nf_ptr ) == 0 ) THEN
         ! "zonal" mean advective heat and salt transport
         pht_adv(:,:) = prt_vj( ztv(:,:,:) )
         pst_adv(:,:) = prt_vj( zsv(:,:,:) )
      ENDIF
#endif

   END SUBROUTINE tra_adv_tvd


   SUBROUTINE nonosc( pbef, paa, pbb, pcc, paft, prdt )
      !!---------------------------------------------------------------------
      !!                    ***  ROUTINE nonosc  ***
      !!     
      !! **  Purpose :   compute monotonic tracer fluxes from the upstream 
      !!       scheme and the before field by a nonoscillatory algorithm 
      !!
      !! **  Method  :   ... ???
      !!       warning : pbef and paft must be masked, but the boundaries
      !!       conditions on the fluxes are not necessary zalezak (1979)
      !!       drange (1995) multi-dimensional forward-in-time and upstream-
      !!       in-space based differencing for fluid
      !!
      !! History :
      !!        !  97-04  (L. Mortier) Original code
      !!        !  00-02  (H. Loukos)  rewritting for opa8
      !!        !  00-10  (M.A Foujols, E. Kestenare)  lateral b.c.
      !!        !  01-03  (E. Kestenare)  add key_passivetrc
      !!        !  01-07  (E. Durand G. Madec)  adapted for T & S
      !!   8.5  !  02-06  (G. Madec)  F90: Free form and module
      !!----------------------------------------------------------------------
      !! * Arguments
      REAL(wp), INTENT( in ) ::   &
         prdt                               ! ???
      REAL(wp), DIMENSION (jpi,jpj,jpk), INTENT( inout ) ::   &
         pbef,                            & ! before field
         paft,                            & ! after field
         paa,                             & ! monotonic flux in the i direction
         pbb,                             & ! monotonic flux in the j direction
         pcc                                ! monotonic flux in the k direction

      !! * Local declarations
      INTEGER ::   ji, jj, jk               ! dummy loop indices
      INTEGER ::   ikm1
      REAL(wp), DIMENSION (jpi,jpj,jpk) ::   zbetup, zbetdo
      REAL(wp) ::   zpos, zneg, zbt, za, zb, zc, zbig, zrtrn, z2dtt
      !!----------------------------------------------------------------------

      zbig = 1.e+40
      zrtrn = 1.e-15

      ! Search local extrema
      ! --------------------
      ! large negative value (-zbig) inside land
      WHERE( tmask(:,:,:) == 0. )
         pbef(:,:,:) = -zbig
         paft(:,:,:) = -zbig
      ENDWHERE 
      ! search maximum in neighbourhood
      DO jk = 1, jpkm1
         ikm1 = MAX(jk-1,1)
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zbetup(ji,jj,jk) = MAX(  pbef(ji  ,jj  ,jk  ), paft(ji  ,jj  ,jk  ),   &
                  &                     pbef(ji-1,jj  ,jk  ), pbef(ji+1,jj  ,jk  ),   &
                  &                     paft(ji-1,jj  ,jk  ), paft(ji+1,jj  ,jk  ),   &
                  &                     pbef(ji  ,jj-1,jk  ), pbef(ji  ,jj+1,jk  ),   &
                  &                     paft(ji  ,jj-1,jk  ), paft(ji  ,jj+1,jk  ),   &
                  &                     pbef(ji  ,jj  ,ikm1), pbef(ji  ,jj  ,jk+1),   &
                  &                     paft(ji  ,jj  ,ikm1), paft(ji  ,jj  ,jk+1)  )
            END DO
         END DO
      END DO
      ! large positive value (+zbig) inside land
      WHERE( tmask(:,:,:) == 0. )
         pbef(:,:,:) = +zbig
         paft(:,:,:) = +zbig
      ENDWHERE
      ! search minimum in neighbourhood
      DO jk = 1, jpkm1
         ikm1 = MAX(jk-1,1)
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zbetdo(ji,jj,jk) = MIN(  pbef(ji  ,jj  ,jk  ), paft(ji  ,jj  ,jk  ),   &
                  &                     pbef(ji-1,jj  ,jk  ), pbef(ji+1,jj  ,jk  ),   &
                  &                     paft(ji-1,jj  ,jk  ), paft(ji+1,jj  ,jk  ),   &
                  &                     pbef(ji  ,jj-1,jk  ), pbef(ji  ,jj+1,jk  ),   &
                  &                     paft(ji  ,jj-1,jk  ), paft(ji  ,jj+1,jk  ),   &
                  &                     pbef(ji  ,jj  ,ikm1), pbef(ji  ,jj  ,jk+1),   &
                  &                     paft(ji  ,jj  ,ikm1), paft(ji  ,jj  ,jk+1)  )
            END DO
         END DO
      END DO

      ! restore masked values to zero
      pbef(:,:,:) = pbef(:,:,:) * tmask(:,:,:)
      paft(:,:,:) = paft(:,:,:) * tmask(:,:,:)
 

      ! 2. Positive and negative part of fluxes and beta terms
      ! ------------------------------------------------------

      DO jk = 1, jpkm1
         z2dtt = prdt * rdttra(jk)
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               ! positive & negative part of the flux
               zpos = MAX( 0., paa(ji-1,jj  ,jk  ) ) - MIN( 0., paa(ji  ,jj  ,jk  ) )   &
                  & + MAX( 0., pbb(ji  ,jj-1,jk  ) ) - MIN( 0., pbb(ji  ,jj  ,jk  ) )   &
                  & + MAX( 0., pcc(ji  ,jj  ,jk+1) ) - MIN( 0., pcc(ji  ,jj  ,jk  ) )
               zneg = MAX( 0., paa(ji  ,jj  ,jk  ) ) - MIN( 0., paa(ji-1,jj  ,jk  ) )   &
                  & + MAX( 0., pbb(ji  ,jj  ,jk  ) ) - MIN( 0., pbb(ji  ,jj-1,jk  ) )   &
                  & + MAX( 0., pcc(ji  ,jj  ,jk  ) ) - MIN( 0., pcc(ji  ,jj  ,jk+1) )
               ! up & down beta terms
               zbt = e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk) / z2dtt
               zbetup(ji,jj,jk) = ( zbetup(ji,jj,jk) - paft(ji,jj,jk) ) / (zpos+zrtrn) * zbt
               zbetdo(ji,jj,jk) = ( paft(ji,jj,jk) - zbetdo(ji,jj,jk) ) / (zneg+zrtrn) * zbt
            END DO
         END DO
      END DO

      ! lateral boundary condition on zbetup & zbetdo   (unchanged sign)
      CALL lbc_lnk( zbetup, 'T', 1. )
      CALL lbc_lnk( zbetdo, 'T', 1. )


      ! 3. monotonic flux in the i direction, i.e. paa
      ! ----------------------------------------------
      DO jk = 1, jpkm1
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zc = paa(ji,jj,jk)
               IF( zc >= 0. ) THEN
                  za = MIN( 1., zbetdo(ji,jj,jk), zbetup(ji+1,jj,jk) )
                  paa(ji,jj,jk) = za * zc
               ELSE
                  zb = MIN( 1., zbetup(ji,jj,jk), zbetdo(ji+1,jj,jk) )
                  paa(ji,jj,jk) = zb * zc
               ENDIF
            END DO
         END DO
      END DO

      ! lateral boundary condition on paa   (changed sign)
      CALL lbc_lnk( paa, 'U', -1. )


      ! 4. monotonic flux in the j direction, i.e. pbb
      ! ----------------------------------------------
      DO jk = 1, jpkm1
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zc = pbb(ji,jj,jk)
               IF( zc >= 0. ) THEN
                  za = MIN( 1., zbetdo(ji,jj,jk), zbetup(ji,jj+1,jk) )
                  pbb(ji,jj,jk) = za * zc
               ELSE
                  zb = MIN( 1., zbetup(ji,jj,jk), zbetdo(ji,jj+1,jk) )
                  pbb(ji,jj,jk) = zb * zc
               ENDIF
            END DO
         END DO
      END DO

      ! lateral boundary condition on pbb   (changed sign)
      CALL lbc_lnk( pbb, 'V', -1. )


      ! monotonic flux in the k direction, i.e. pcc
      ! -------------------------------------------
      DO jk = 2, jpkm1
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zc = pcc(ji,jj,jk)
               IF( zc >= 0. ) THEN
                  za = MIN( 1., zbetdo(ji,jj,jk), zbetup(ji,jj,jk-1) )
                  pcc(ji,jj,jk) = za * zc
               ELSE
                  zb = MIN( 1., zbetup(ji,jj,jk), zbetdo(ji,jj,jk-1) )
                  pcc(ji,jj,jk) = zb * zc
               ENDIF
            END DO
         END DO
      END DO

      ! lateral boundary condition on pcc   (unchanged sign)
      CALL lbc_lnk( pcc, 'W', 1. )

   END SUBROUTINE nonosc

   !!======================================================================
END MODULE traadv_tvd
