MODULE traldf_bilap
   !!==============================================================================
   !!                   ***  MODULE  traldf_bilap  ***
   !! Ocean active tracers:  horizontal component of the lateral tracer mixing trend
   !!==============================================================================

   !!----------------------------------------------------------------------
   !!   tra_ldf_bilap : update the tracer trend with the horizontal diffusion
   !!                   using a iso-level biharmonic operator
   !!----------------------------------------------------------------------
   !! * Modules used
   USE oce             ! ocean dynamics and active tracers
   USE dom_oce         ! ocean space and time domain
   USE ldftra_oce     ! ocean tracer   lateral physics
   USE trdtra_oce     ! ocean active tracer trend 
   USE in_out_manager  ! I/O manager
   USE ldfslp          ! iso-neutral slopes 
   USE lbclnk          ! ocean lateral boundary conditions (or mpp link)

   IMPLICIT NONE
   PRIVATE

   !! * Routine accessibility
   PUBLIC tra_ldf_bilap   ! routine called by step.F90

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "ldftra_substitute.h90"
#  include "ldfeiv_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LODYC-IPSL (2003)
   !!----------------------------------------------------------------------

CONTAINS
   
   SUBROUTINE tra_ldf_bilap( kt )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE tra_ldf_bilap  ***
      !!
      !! ** Purpose :   Compute the before horizontal tracer (t & s) diffusive 
      !!      trend and add it to the general trend of tracer equation.
      !!
      !! ** Method  :   4th order diffusive operator along model level surfaces 
      !!      evaluated using before fields (forward time scheme). The hor.
      !!      diffusive trends of temperature (idem for salinity) is given by:
      !!       * s-coordinate ('key_s_coord' defined), the vertical scale 
      !!      factors e3. are inside the derivatives:
      !!      Laplacian of tb:
      !!         zlt   = 1/(e1t*e2t*e3t) {  di-1[ e2u*e3u/e1u di(tb) ]
      !!                                  + dj-1[ e1v*e3v/e2v dj(tb) ]  }
      !!      Multiply by the eddy diffusivity coef. and insure lateral bc:
      !!        zlt   = ahtt * zlt
      !!        call to lbc_lnk
      !!      Bilaplacian (laplacian of zlt):
      !!         difft = 1/(e1t*e2t*e3t) {  di-1[ e2u*e3u/e1u di(zlt) ]
      !!                                  + dj-1[ e1v*e3v/e2v dj(zlt) ]  }
      !!       * z-coordinate (default key), e3t=e3u=e3v, the trend becomes:
      !!      Laplacian of tb:
      !!         zlt   = 1/(e1t*e2t) {  di-1[ e2u/e1u di(tb) ]
      !!                              + dj-1[ e1v/e2v dj(tb) ] }
      !!      Multiply by the eddy diffusivity coef. and insure lateral bc:
      !!        zlt   = ahtt * zlt
      !!        call to lbc_lnk
      !!      Bilaplacian (laplacian of zlt):
      !!         difft = 1/(e1t*e2t) {  di-1[ e2u/e1u di(zlt) ]
      !!                              + dj-1[ e1v/e2v dj(zlt) ]  }
      !!
      !!      Add this trend to the general trend (ta,sa):
      !!         (ta,sa) = (ta,sa) + ( difft , diffs )
      !!
      !! ** Action : - Update (ta,sa) arrays with the before iso-level
      !!               biharmonic mixing trend.
      !!             - Save the trends in (ttrd,strd) ('key_diatrends')
      !!
      !! History :
      !!        !  91-11  (G. Madec)  Original code
      !!        !  93-03  (M. Guyon)  symetrical conditions
      !!        !  95-11  (G. Madec)  suppress volumetric scale factors
      !!        !  96-01  (G. Madec)  statement function for e3
      !!        !  96-01  (M. Imbard)  mpp exchange
      !!        !  97-07  (G. Madec)  optimization, and ahtt
      !!   8.5  !  02-08  (G. Madec)  F90: Free form and module
      !!----------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( in ) ::   kt       ! ocean time-step index

      !! * Local declarations
      INTEGER ::   ji, jj, jk             ! dummy loop indices
#if defined key_partial_steps
      INTEGER ::   iku, ikv               ! temporary integers
#endif
      REAL(wp) ::   zta, zsa    ! temporary scalars
      REAL(wp), DIMENSION(jpi,jpj) ::   & 
         zeeu, zeev, zbtr,             &  ! workspace
         zlt, ztu, ztv,                &
         zls, zsu, zsv
      !!----------------------------------------------------------------------

      IF( kt == nit000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'tra_ldf_bilap : iso-level biharmonic operator'
         IF(lwp) WRITE(numout,*) '~~~~~~~~~~~~~'
      ENDIF
      !                                                ! ===============
      DO jk = 1, jpkm1                                 ! Horizontal slab
         !                                             ! ===============

         ! 0. Initialization of metric arrays (for z- or s-coordinates)
         ! ----------------------------------

         DO jj = 1, jpjm1
            DO ji = 1, fs_jpim1   ! vector opt.
#if defined key_s_coord || defined key_partial_steps
               ! s-coordinates, vertical scale factor are used
               zbtr(ji,jj) = 1. / ( e1t(ji,jj)*e2t(ji,jj)*fse3t(ji,jj,jk) )
               zeeu(ji,jj) = e2u(ji,jj) * fse3u(ji,jj,jk) / e1u(ji,jj) * umask(ji,jj,jk)
               zeev(ji,jj) = e1v(ji,jj) * fse3v(ji,jj,jk) / e2v(ji,jj) * vmask(ji,jj,jk)
#else
               ! z-coordinates, no vertical scale factors
               zbtr(ji,jj) = 1. / ( e1t(ji,jj)*e2t(ji,jj) )
               zeeu(ji,jj) = e2u(ji,jj) / e1u(ji,jj) * umask(ji,jj,jk)
               zeev(ji,jj) = e1v(ji,jj) / e2v(ji,jj) * vmask(ji,jj,jk)
#endif
            END DO
         END DO


         ! 1. Laplacian
         ! ------------

         ! First derivative (gradient)
         DO jj = 1, jpjm1
            DO ji = 1, fs_jpim1   ! vector opt.
               ztu(ji,jj) = zeeu(ji,jj) * ( tb(ji+1,jj  ,jk) - tb(ji,jj,jk) )
               zsu(ji,jj) = zeeu(ji,jj) * ( sb(ji+1,jj  ,jk) - sb(ji,jj,jk) )
               ztv(ji,jj) = zeev(ji,jj) * ( tb(ji  ,jj+1,jk) - tb(ji,jj,jk) )
               zsv(ji,jj) = zeev(ji,jj) * ( sb(ji  ,jj+1,jk) - sb(ji,jj,jk) )
            END DO
         END DO
#if defined key_partial_steps
         DO jj = 1, jpj-1
            DO ji = 1, jpi-1
               ! last level
               iku = MIN ( mbathy(ji,jj), mbathy(ji+1,jj  ) ) - 1
               ikv = MIN ( mbathy(ji,jj), mbathy(ji  ,jj+1) ) - 1
               IF( iku == jk ) THEN
                  ztu(ji,jj) = zeeu(ji,jj) * gtu(ji,jj)
                  zsu(ji,jj) = zeeu(ji,jj) * gsu(ji,jj)
               ENDIF
               IF( ikv == jk ) THEN
                  ztv(ji,jj) = zeev(ji,jj) * gtv(ji,jj)
                  zsv(ji,jj) = zeev(ji,jj) * gsv(ji,jj)
               ENDIF
            END DO
         END DO
#endif

         ! Second derivative (divergence)
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zlt(ji,jj) = zbtr(ji,jj) * (  ztu(ji,jj) - ztu(ji-1,jj) + ztv(ji,jj) - ztv(ji,jj-1)  )
               zls(ji,jj) = zbtr(ji,jj) * (  zsu(ji,jj) - zsu(ji-1,jj) + zsv(ji,jj) - zsv(ji,jj-1)  )
            END DO
         END DO

         ! Multiply by the eddy diffusivity coefficient
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zlt(ji,jj) = fsahtt(ji,jj,jk) * zlt(ji,jj)
               zls(ji,jj) = fsahtt(ji,jj,jk) * zls(ji,jj)
            END DO
         END DO

         ! Lateral boundary conditions on the laplacian (zlt,zls)   (unchanged sgn)
         CALL lbc_lnk( zlt, 'T', 1. )   ;    CALL lbc_lnk( zls, 'T', 1. )

         ! 2. Bilaplacian
         ! --------------

         ! third derivative (gradient)
         DO jj = 1, jpjm1
            DO ji = 1, fs_jpim1   ! vector opt.
               ztu(ji,jj) = zeeu(ji,jj) * ( zlt(ji+1,jj  ) - zlt(ji,jj) )
               zsu(ji,jj) = zeeu(ji,jj) * ( zls(ji+1,jj  ) - zls(ji,jj) )
               ztv(ji,jj) = zeev(ji,jj) * ( zlt(ji  ,jj+1) - zlt(ji,jj) )
               zsv(ji,jj) = zeev(ji,jj) * ( zls(ji  ,jj+1) - zls(ji,jj) )
            END DO
         END DO

         ! fourth derivative (divergence) and add to the general tracer trend
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               ! horizontal diffusive trends
               zta = zbtr(ji,jj) * (  ztu(ji,jj) - ztu(ji-1,jj) + ztv(ji,jj) - ztv(ji,jj-1)  )
               zsa = zbtr(ji,jj) * (  zsu(ji,jj) - zsu(ji-1,jj) + zsv(ji,jj) - zsv(ji,jj-1)  )
               ! add it to the general tracer trends
               ta(ji,jj,jk) = ta(ji,jj,jk) + zta
               sa(ji,jj,jk) = sa(ji,jj,jk) + zsa
#if defined key_trdtra || defined key_trdmld
               ! save the horizontal diffusive trends
               ttrd(ji,jj,jk,3) = zta
               strd(ji,jj,jk,3) = zsa
#endif
            END DO
         END DO
         !                                             ! ===============
      END DO                                           ! Horizontal slab
      !                                                ! ===============

#if defined key_diaptr
      ! "zonal" mean lateral diffusive heat and salt transport
      == >> forced bug NOT implemented, ztv, zsv not 3D arrays
      IF( MOD( kt, nf_ptr ) == 0 ) THEN
# if defined key_s_coord || defined key_partial_steps
         pht_ldf(:,:) = prt_vj( ztv(:,:,:) )
         pst_ldf(:,:) = prt_vj( zsv(:,:,:) )
# else
         DO jk = 1, jpkm1
            DO jj = 2, jpjm1
               DO ji = fs_2, fs_jpim1   ! vector opt.
                 ztv(ji,jj,jk) = ztv(ji,jj,jk) * fse3v(ji,jj,jk)
                 zsv(ji,jj,jk) = zsv(ji,jj,jk) * fse3v(ji,jj,jk)
               END DO
            END DO
         END DO
         pht_ldf(:,:) = prt_vj( ztv(:,:,:) )
         pst_ldf(:,:) = prt_vj( zsv(:,:,:) )
# endif
      ENDIF
#endif

   END SUBROUTINE tra_ldf_bilap

   !!==============================================================================
END MODULE traldf_bilap
